// File created from ../../../src/lib/eval/eval_messages.mes on Mon Jun 22 2020 17:21

#ifndef EVAL_MESSAGES_H
#define EVAL_MESSAGES_H

#include <log/message_types.h>

namespace isc {
namespace dhcp {

extern const isc::log::MessageID EVAL_DEBUG_AND;
extern const isc::log::MessageID EVAL_DEBUG_CONCAT;
extern const isc::log::MessageID EVAL_DEBUG_EQUAL;
extern const isc::log::MessageID EVAL_DEBUG_HEXSTRING;
extern const isc::log::MessageID EVAL_DEBUG_IFELSE_FALSE;
extern const isc::log::MessageID EVAL_DEBUG_IFELSE_TRUE;
extern const isc::log::MessageID EVAL_DEBUG_IPADDRESS;
extern const isc::log::MessageID EVAL_DEBUG_MEMBER;
extern const isc::log::MessageID EVAL_DEBUG_NOT;
extern const isc::log::MessageID EVAL_DEBUG_OPTION;
extern const isc::log::MessageID EVAL_DEBUG_OR;
extern const isc::log::MessageID EVAL_DEBUG_PKT;
extern const isc::log::MessageID EVAL_DEBUG_PKT4;
extern const isc::log::MessageID EVAL_DEBUG_PKT6;
extern const isc::log::MessageID EVAL_DEBUG_RELAY6;
extern const isc::log::MessageID EVAL_DEBUG_RELAY6_RANGE;
extern const isc::log::MessageID EVAL_DEBUG_STRING;
extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING;
extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING_EMPTY;
extern const isc::log::MessageID EVAL_DEBUG_SUBSTRING_RANGE;
extern const isc::log::MessageID EVAL_DEBUG_SUB_OPTION;
extern const isc::log::MessageID EVAL_DEBUG_SUB_OPTION_NO_OPTION;
extern const isc::log::MessageID EVAL_DEBUG_TOHEXSTRING;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_DATA;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_DATA_NOT_FOUND;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_ENTERPRISE_ID;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_ENTERPRISE_ID_MISMATCH;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_EXISTS;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_CLASS_NO_OPTION;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_ENTERPRISE_ID;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_ENTERPRISE_ID_MISMATCH;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_EXISTS;
extern const isc::log::MessageID EVAL_DEBUG_VENDOR_NO_OPTION;
extern const isc::log::MessageID EVAL_RESULT;

} // namespace dhcp
} // namespace isc

#endif // EVAL_MESSAGES_H
